? mycore
Index: channels/chan_zap.c
===================================================================
RCS file: /usr/cvsroot/asterisk/channels/chan_zap.c,v
retrieving revision 1.490
diff -u -r1.490 chan_zap.c
--- channels/chan_zap.c	19 Aug 2005 04:23:23 -0000	1.490
+++ channels/chan_zap.c	30 Aug 2005 02:14:55 -0000
@@ -257,6 +257,7 @@
 static int cur_rxwink = -1;
 static int cur_rxflash = -1;
 static int cur_debounce = -1;
+static int cur_priexclusive = 0;
 
 static int priindication_oob = 0;
 
@@ -543,6 +544,7 @@
 	unsigned int permcallwaiting:1;
 	unsigned int permhidecallerid:1;		/* Whether to hide our outgoing caller ID or not */
 	unsigned int priindication_oob:1;
+	unsigned int priexclusive:1;
 	unsigned int pulse:1;
 	unsigned int pulsedial:1;			/* whether a pulse dial phone is detected */
 	unsigned int restrictcid:1;			/* Whether restrict the callerid -> only send ANI */
@@ -1875,6 +1877,8 @@
 		int dp_strip;
 		int prilocaldialplan;
 		int ldp_strip;
+		int exclusive;
+
 		c = strchr(dest, '/');
 		if (c)
 			c++;
@@ -1931,8 +1935,18 @@
 			pri_set_crv(p->pri->pri, p->call, p->channel, 0);
 		}
 		p->digital = IS_DIGITAL(ast->transfercapability);
-		pri_sr_set_channel(sr, p->bearer ? PVT_TO_CHANNEL(p->bearer) : PVT_TO_CHANNEL(p), 
-								p->pri->nodetype == PRI_NETWORK ? 0 : 1, 1);
+		/* Add support for exclusive override */
+		if (p->priexclusive)
+			exclusive = 1;
+		else {
+		/* otherwise, traditional behavior */
+			if (p->pri->nodetype == PRI_NETWORK)
+				exclusive = 0;
+			else
+				exclusive = 1;
+		}
+		
+		pri_sr_set_channel(sr, p->bearer ? PVT_TO_CHANNEL(p->bearer) : PVT_TO_CHANNEL(p), exclusive, 1);
 		pri_sr_set_bearer(sr, p->digital ? PRI_TRANS_CAP_DIGITAL : ast->transfercapability, 
 					(p->digital ? -1 : 
 						((p->law == ZT_LAW_ALAW) ? PRI_LAYER_1_ALAW : PRI_LAYER_1_ULAW)));
@@ -4668,7 +4682,7 @@
 			ast_log(LOG_DEBUG,"Received AST_CONTROL_PROGRESS on %s\n",chan->name);
 #ifdef ZAPATA_PRI
 			p->digital = 0;	/* Digital-only calls isn't allows any inband progress messages */
-			if (!p->proceeding && p->sig==SIG_PRI && p->pri && !p->outgoing) {
+			if ((p->proceeding < 2) && p->sig==SIG_PRI && p->pri && !p->outgoing) {
 				if (p->pri->pri) {		
 					if (!pri_grab(p, p->pri)) {
 						pri_progress(p->pri->pri,p->call, PVT_TO_CHANNEL(p), 1);
@@ -6925,6 +6939,7 @@
 		tmp->restrictcid = restrictcid;
 		tmp->use_callingpres = use_callingpres;
 		tmp->priindication_oob = priindication_oob;
+		tmp->priexclusive = cur_priexclusive;
 		tmp->emdigitwait = cur_emdigitwait;
 		if (tmp->usedistinctiveringdetection) {
 			if (!tmp->use_callerid) {
@@ -10406,6 +10421,8 @@
 				else
 					ast_log(LOG_WARNING, "'%s' is not a valid pri indication value, should be 'inband' or 'outofband' at line %d\n",
 						v->value, v->lineno);
+			} else if (!strcasecmp(v->name, "priexclusive")) {
+				cur_priexclusive = ast_true(v->value);
 			} else if (!strcasecmp(v->name, "internationalprefix")) {
 				ast_copy_string(internationalprefix, v->value, sizeof(internationalprefix));
 			} else if (!strcasecmp(v->name, "nationalprefix")) {
Index: configs/zapata.conf.sample
===================================================================
RCS file: /usr/cvsroot/asterisk/configs/zapata.conf.sample,v
retrieving revision 1.50
diff -u -r1.50 zapata.conf.sample
--- configs/zapata.conf.sample	8 Aug 2005 00:08:48 -0000	1.50
+++ configs/zapata.conf.sample	30 Aug 2005 02:14:55 -0000
@@ -124,6 +124,10 @@
 ;
 ; priindication = outofband
 ;
+; If you need to override the existing channels selection routine and force all
+; PRI channels to be marked as exclusively selected, set this to yes.
+; priexclusive = yes
+;
 ; ISDN Timers
 ; All of the ISDN timers and counters that are used are configurable.  Specify 
 ; the timer name, and its value (in ms for timers)
